@@ -5,71 +5,17 @@ from typing import Optional, List
 
 import torch as tr
 from torch import Tensor, nn
-from torch.jit import Final
 
 from neutone_sdk import WaveformToWaveformMetadata
-from neutone_sdk.sandwich import InterpolationResampler, ChannelNormalizerSandwich
 from neutone_sdk.constants import DEFAULT_DAW_SR, DEFAULT_DAW_BS
+from neutone_sdk.queues import CircularInplaceTensorQueue
+from neutone_sdk.sandwich import ChannelNormalizerSandwich, InplaceInterpolationResampler
 
 logging.basicConfig()
 log = logging.getLogger(__name__)
 log.setLevel(level=os.environ.get("LOGLEVEL", "INFO"))
 
 
-class InplaceTensorAudioQueue:
-    def __init__(self, n_ch: int, queue_len: int, use_debug_mode: bool = True) -> None:
-        self.use_debug_mode = use_debug_mode
-        self.queue_len = queue_len
-        self.queue = tr.zeros((n_ch, queue_len))
-        self.tmp_queue = tr.zeros((n_ch, queue_len))
-        self.size = 0
-
-    def push(self, x: Tensor) -> None:
-        if self.use_debug_mode:
-            assert x.ndim == self.queue.ndim
-            assert x.size(0) == self.queue.size(0)
-        in_n = x.size(1)
-        if self.use_debug_mode:
-            assert self.size + in_n < self.queue_len
-        self.queue[:, self.size:self.size + in_n] = x
-        self.size += in_n
-
-    def pop(self, out: Tensor) -> int:
-        if self.use_debug_mode:
-            assert out.ndim == self.queue.ndim
-            assert out.size(0) == self.queue.size(0)
-        out_n = out.size(1)
-        if self.use_debug_mode:
-            assert out_n <= self.queue_len
-        out[:, :] = self.queue[:, 0:out_n]
-        return self.remove(out_n)
-
-    def head(self, n: int) -> Tensor:
-        if self.use_debug_mode:
-            assert 0 < n <= self.queue_len
-        return self.queue[:, 0:n]
-
-    def remove(self, out_n: int) -> int:
-        removed_n = min(self.size, out_n)
-        if removed_n > 0:
-            remaining_n = self.size - removed_n
-            # This avoids allocating memory like tr.roll does
-            self.tmp_queue[:, :remaining_n] = self.queue[:, removed_n:removed_n + remaining_n]
-            self.queue[:, :remaining_n] = self.tmp_queue[:, :remaining_n]
-            self.queue[:, remaining_n:removed_n + remaining_n] = 0
-        self.size -= removed_n
-        return removed_n
-
-    def is_empty(self) -> bool:
-        return self.size == 0
-
-    def reset(self) -> None:
-        self.queue.fill_(0)
-        self.tmp_queue.fill_(0)
-        self.size = 0
-
-
-# TODO(cm): add support for crossfading
 class SampleQueueWrapper(nn.Module):
     def __init__(self,
                  w2w_base: "WaveformToWaveformBase",
@@ -87,11 +33,24 @@ class SampleQueueWrapper(nn.Module):
         super().__init__()
         self.use_debug_mode = use_debug_mode
 
+        self.w2w_base = w2w_base
+        self.in_n_ch = 1 if self.is_input_mono() else 2
+        self.out_n_ch = 1 if self.is_output_mono() else 2
+
         self.channel_normalizer = ChannelNormalizerSandwich(use_debug_mode=use_debug_mode)
         # TODO(cm): switch to a more robust resampling method that prevents aliasing
-        self.resample_sandwich = InterpolationResampler(daw_sr, daw_sr)  # Tmp sample rate values
-        self.params_resample_sandwich = InterpolationResampler(daw_sr, daw_sr)  # Tmp sample rate values
-        self.w2w_base = w2w_base
+        self.resample_sandwich = InplaceInterpolationResampler(self.in_n_ch,
+                                                               self.out_n_ch,
+                                                               daw_sr,
+                                                               daw_sr,  # Tmp sample rate values
+                                                               daw_bs,
+                                                               use_debug_mode=use_debug_mode)
+        self.params_resample_sandwich = InplaceInterpolationResampler(self.w2w_base.MAX_N_PARAMS,
+                                                                      self.w2w_base.MAX_N_PARAMS,
+                                                                      daw_sr,
+                                                                      daw_sr,  # Tmp sample rate value
+                                                                      daw_bs,
+                                                                      use_debug_mode=use_debug_mode)
 
         self.daw_sr = daw_sr
         self.model_sr = model_sr
@@ -101,9 +60,6 @@ class SampleQueueWrapper(nn.Module):
         self.is_queue_saturated = False
         self.saturation_n = None
 
-        self.in_n_ch = 1 if self.is_input_mono() else 2
-        self.out_n_ch = 1 if self.is_output_mono() else 2
-
         self.in_queue = None
         self.params_queue = None
         self.out_queue = None
@@ -211,6 +167,11 @@ class SampleQueueWrapper(nn.Module):
         self.w2w_base.prepare_for_inference()
         self.use_debug_mode = False
         self.channel_normalizer.use_debug_mode = False
+        self.resample_sandwich.use_debug_mode = False
+        self.params_resample_sandwich.use_debug_mode = False
+        self.in_queue.use_debug_mode = False
+        self.params_queue.use_debug_mode = False
+        self.out_queue.use_debug_mode = False
         self.eval()
 
     def _forward(self, resampled_x: Tensor, params: Optional[Tensor] = None) -> None:
@@ -253,7 +214,7 @@ class SampleQueueWrapper(nn.Module):
         # if self.is_queue_saturated and out_popped_n < x.shape[1]:
         #     log.warning('queue is starved')
 
-        x = self.resample_sandwich.process_out(self.io_out_buffer, in_n)
+        x = self.resample_sandwich.process_out(self.io_out_buffer)
         if self.use_debug_mode:
             assert x.size(1) == in_n
         x = self.channel_normalizer(x, is_daw_mono, self.daw_buffer)
@@ -273,7 +234,7 @@ class SampleQueueWrapper(nn.Module):
             out_popped_n = self.out_queue.pop(self.io_out_buffer)
             if self.use_debug_mode:
                 assert out_popped_n == self.io_bs
-            x = self.resample_sandwich.process_out(self.io_out_buffer, self.daw_bs)
+            x = self.resample_sandwich.process_out(self.io_out_buffer)
             x = self.channel_normalizer(x, is_daw_mono, self.daw_buffer)
             if self.use_debug_mode:
                 assert x.size(1) == self.daw_bs
@@ -331,8 +292,8 @@ class SampleQueueWrapper(nn.Module):
 
         io_bs = self.calc_resampled_buffer_size(daw_sr, model_sr, daw_bs)
 
-        self.resample_sandwich.set_sample_rates(daw_sr, model_sr)
-        self.params_resample_sandwich.set_sample_rates(daw_sr, model_sr)
+        self.resample_sandwich.set_sample_rates(daw_sr, model_sr, daw_bs)
+        self.params_resample_sandwich.set_sample_rates(daw_sr, model_sr, daw_bs)
         self.daw_sr = daw_sr
         self.model_sr = model_sr
 
@@ -347,10 +308,15 @@ class SampleQueueWrapper(nn.Module):
         self.io_bs = io_bs
         self.model_bs = model_bs
 
-        self.in_queue = InplaceTensorAudioQueue(self.in_n_ch, (2 * self.io_bs) + self.model_bs)
-        self.params_queue = InplaceTensorAudioQueue(
-            self.w2w_base.MAX_N_PARAMS, (2 * self.io_bs) + self.model_bs)
-        self.out_queue = InplaceTensorAudioQueue(self.out_n_ch, (2 * self.io_bs) + self.model_bs)
+        self.in_queue = CircularInplaceTensorQueue(self.in_n_ch,
+                                                   (2 * self.io_bs) + self.model_bs,
+                                                   use_debug_mode=self.use_debug_mode)
+        self.params_queue = CircularInplaceTensorQueue(self.w2w_base.MAX_N_PARAMS,
+                                                       (2 * self.io_bs) + self.model_bs,
+                                                       use_debug_mode=self.use_debug_mode)
+        self.out_queue = CircularInplaceTensorQueue(self.out_n_ch,
+                                                    (2 * self.io_bs) + self.model_bs,
+                                                    use_debug_mode=self.use_debug_mode)
 
         self.daw_buffer = tr.zeros((2, self.daw_bs))
         self.model_in_buffer = tr.zeros((self.in_n_ch, self.model_bs))
